/*
 * Copyright (c) 2010, 2014, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the LICENSE file that accompanied this code.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */
package jdk.nashorn.internal.runtime;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.security.AccessController;
import java.security.PrivilegedAction;
import java.util.zip.InflaterInputStream;
import jdk.nashorn.internal.ir.FunctionNode;

/**
 * This static utility class performs deserialization of FunctionNode ASTs from a byte array.
 * The format is a standard Java serialization stream, deflated.
 */
final class AstDeserializer {
    static FunctionNode deserialize(final byte[] serializedAst) {
        return AccessController.doPrivileged(new PrivilegedAction<FunctionNode>() {
            @Override
            public FunctionNode run() {
                try {
                    return (FunctionNode)new ObjectInputStream(new InflaterInputStream(
                        new ByteArrayInputStream(serializedAst))).readObject();
                } catch (final ClassNotFoundException | IOException e) {
                    // This is internal, can't happen
                    throw new AssertionError("Unexpected exception deserializing function", e);
                }
            }
        });
    }
}
